#!/bin/bash

# Script Bash - install.sh
# Auteur : Cédric Abonnel

# Description : Script d'installation des exécutables dans le répertoire '~/.local'"

# Version 23.12.1.3
# - Amélioration des logs
# - process_deployment_files optmisie les opérations de déployement

# Version 23.12.1.2
# - Fonctionnalité de purge des logs ajoutée.

# Version 23.12.1.1
# - Messages des logs améliorés avec plus de détail avec 'cp'.


# Spécifier le chemin du fichier journal
log_dir="$HOME/log"
log_file="${log_dir}/a5l-scripts_bash-$(date '+%Y%m%d-%H%M%S')-$$.log"

# Fonction pour afficher un message d'erreur et quitter le script en cas d'erreur
error() {
    local error_message="$1"
    log "ERREUR: $error_message"
    echo "Erreur: $error_message"
    exit 1
}

# Fonction pour enregistrer les messages de journal
log() {
    if [ -n "$1" ]; then
      echo "$(date '+%Y-%m-%d %H:%M:%S') - $$ - $1" >> "$log_file"
      echo "$1"
    fi
}

create_dir() {
    # S'assurer que le répertoire de destination existe, sinon, le créer
    log "Tentative de création du dossier $1"
    if [ ! -d "$1" ]; then
      mkdir -p "$1" || error "Impossible de créer le répertoire $1"
      log " _ Le répertoire $1 a été créé."
    else
      log " _ Le répertoire $1 existe déjà."
    fi

}


add_uninstall() {
    ## Create or update uninstall script

    uninstall_list=$HOME/.config/a5l_scripts-bash_uninstall-list
    test -f "${uninstall_list}" || touch "${uninstall_list}"

    local fichier_a_supprimer="$1"

       # Vérifier si le fichier de suppression existe déjà dans uninstall_script
    if ! grep -q "$fichier_a_supprimer" "$uninstall_list"; then
        # Ajouter le fichier à uninstall_list
        echo "$fichier_a_supprimer" >> "$uninstall_list"
        log "- Le fichier '$fichier_a_supprimer' ajouté à la liste de fichiers installés"
    fi
}


## Fonction de Traitement ##
process_deployment_files() {
    local destination="$1"
    local source="$2"
    local file_desc="$3"

    log "### Déploiement des fichiers pour $destination"

    # S'assurer que le répertoire de destination existe, sinon, le créer
    create_dir "$destination"

    # Copier sélectivement les fichiers du dépôt Git local vers $destination en préservant la structure
    log "Vérification de l'existence de '$file_desc'"
    
    # Vérifier si le fichier "$file_desc" existe
    if [ -e "$file_desc" ]; then
        # Récupérer des fichiers installés
        fileslist_local=$(cat "$file_desc")

        # Copie des fichiers listés dans $file_desc
        log "Copie des fichiers listés dans $file_desc."

        # Copier les fichiers en supprimant les extensions
        for file in $fileslist_local; do
            # debut Supprimer l'extension du nom de fichier si c'est un Bash Script
            file_type=$(file -b "$source/$file")
            if [[ "$file_type" == *"Bourne-Again shell script"* ]]; then
              log "$file est un fichier Bash."
              file_dest="${file%.*}"
              log "Nom de fichier sans extension : $file_dest"
            else
              file_dest="${file}"
            fi
            # fin
            cp -v "$source/$file" "$destination/$file_dest" 2>&1 | while read -r line; do
                log "- $line"
            done
            add_uninstall "$destination/$file_dest"
        done
    else
        error "Le fichier '$file_desc' n'existe pas. Votre dépôt Git local n'est pas complet ou quelque chose s'est mal passé."
    fi

    # Affiche un message de confirmation
    log "Les fichiers du dépôt Git local ont été copiés vers $destination_dir avec succès."
}


purge_old_logs(){
  find "$log_dir" -type f -name "a5l-scripts_bash-*.log" -mtime +10 -exec rm {} \;
  find "$log_dir" -type f -name "a5l-scripts_bash_prep-*.log" -mtime +10 -exec rm {} \;
}

log "Debut du script"

## Post traitement : log

# S'assurer que le répertoire de destination existe, sinon, le créer
create_dir "${log_dir}"

# Purger les les logs
purge_old_logs



#### .local/share/doc ####

## Declarations ##

# Spécifier le chemin du répertoire destination
destination_dir="$HOME/.local/share/doc"

# Spécifier le chemin du répertoire du dépôt Git local
source_dir="local/share/doc"

file_local_desc=.config/files_local-share-doc

## Traitement ##
log "## Debut du traitement pour $destination_dir"

process_deployment_files "$destination_dir" "$source_dir" "$file_local_desc"




#### .local/share/man ####

## Declarations ##

# Spécifier le chemin du répertoire
destination_dir="$HOME/.local/share/man"

# Spécifier le chemin du répertoire du dépôt Git local
source_dir="local/share/man"

file_local_desc=.config/files_local-share-man

## Traitement ##
log "## Debut du traitement pour $destination_dir"

log "### Debut du Traitement pour la gestion du PATH avec $destination_dir"
# Vérifier si le répertoire destination est déjà dans le PATH
if [[ ! ":$(manpath):" == *":$destination_dir:"* ]]; then
  # Ajouter le répertoire destination au PATH dans le fichier de configuration de session de l'utilisateur
  echo 'export MANPATH="$(manpath):'"$destination_dir"'"' >> ~/.bashrc
  source ~/.bashrc
  log "  $destination_dir a été ajouté au MANPATH dans le fichier de configuration de session de l'utilisateur."
fi

process_deployment_files "$destination_dir" "$source_dir" "$file_local_desc"



#### .local/bin ####

## Declarations ##

# Spécifier le chemin du répertoire
destination_dir="$HOME/.local/bin"

# Spécifier le chemin du répertoire du dépôt Git local
source_dir="local/bin"

file_local_desc=.config/files_local-bin

## Traitement ##
log "## Debut du traitement pour $destination_dir"

# S'assurer que le répertoire de destination existe, sinon, le créer
create_dir "$destination_dir"

log "### Debut du Traitement pour la gestion du PATH avec $destination_dir"
# Vérifier si le répertoire destination est déjà dans le PATH
if [[ ! ":$PATH:" == *":$destination_dir:"* ]]; then
  # Ajouter le répertoire destination au PATH dans le fichier de configuration de session de l'utilisateur
  echo 'export PATH="$PATH:'"$destination_dir"'"' >> ~/.bashrc
  source ~/.bashrc
  log "  $destination_dir a été ajouté au PATH dans le fichier de configuration de session de l'utilisateur."
fi

## Traitement ##
process_deployment_files "$destination_dir" "$source_dir" "$file_local_desc"

log "## Fin du script. Bonne continuation."

